SystemOrganization addCategory: #'Croquet-Menus'!

TGroup subclass: #TMenuFrame
	instanceVariableNames: 'labelFrame menuForm menuFormExtent menuTxtr menuItems alignment selectedItem subMenu parentMenu focused menuStyle selectedPoint cameraNorm'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Croquet-Menus'!

!TMenuFrame commentStamp: '<historical>' prior: 0!
(c) 2006 Qwaq Inc. TMenuFrame is a TFrame for displaying a menu.

Instance variables:
	labelFrame	<TFrame>	The menu label (used for dragging)
	menuForm	<TForm>		The menu form (used for rendering)
	menuFormExtent	<Point>	The actual extent of the menu form (non-power-of-two)
	menuTxtr	<TTexture>	The texture displayed for the menu.
	menuItems	<Array of: TMenuItem>	The menu items.
	alignment	<Symbol>	A symbol describing how to align the the menu inside the frame.
	selectedItem	<TMenuItem | nil>	The currently selected menu item.
	subMenu	<TMenuFrame | nil>	The currently active sub menu (if any).
	parentMenu	<TMenuFrame | nil>	The parent menu if this is a submenu.
	focused 	<Boolean>	True if the pointer is over this menu.
	menuStyle	<TMenuStyle> The style attributes for the menu.
	selectedPoint <Vector3>	Dragging only.
	cameraNorm <Vector3>	Dragging only.
!

!TMenuFrame methodsFor: 'constructing' stamp: 'ar 7/31/2006 11:18'!
add: aString action: anAction
	^self add: aString target: anAction action: #value! !

!TMenuFrame methodsFor: 'constructing' stamp: 'ar 7/31/2006 11:19'!
add: aString subMenu: aMenu
	| item |
	item := self addItem: aString.
	item subMenu: aMenu.
	^item! !

!TMenuFrame methodsFor: 'constructing' stamp: 'ar 7/31/2006 11:19'!
add: aString target: anObject action: aSelector
	^self add: aString target: anObject action: aSelector arguments: #()! !

!TMenuFrame methodsFor: 'constructing' stamp: 'ar 7/31/2006 11:19'!
add: aString target: anObject action: aSelector argument: arg
	^self add: aString target: anObject action: aSelector arguments: {arg}! !

!TMenuFrame methodsFor: 'constructing' stamp: 'ar 9/25/2006 13:52'!
add: aString target: anObject action: aSelector arguments: args
	| item |
	item := self addItem: aString.
	item action: (TMenuAction receiver: anObject selector: aSelector arguments: args).
	(anObject respondsTo: aSelector) ifFalse:[item enabled: false].
	^item! !

!TMenuFrame methodsFor: 'constructing' stamp: 'ar 7/31/2006 11:19'!
add: aString target: anObject action: aSelector checked: aBool
	| item |
	item := self add: aString target: anObject action: aSelector.
	item checked: aBool.
	^item! !

!TMenuFrame methodsFor: 'constructing' stamp: 'ar 7/31/2006 11:19'!
add: aString target: anObject action: aSelector disabled: aBool
	| item |
	item := self add: aString target: anObject action: aSelector.
	item disabled: aBool.
	^item! !

!TMenuFrame methodsFor: 'constructing' stamp: 'ar 7/31/2006 13:01'!
addItem: aString
	| item |
	item := TMenuItem new.
	item label: aString.
	self addMenuItem: item.
	^item! !

!TMenuFrame methodsFor: 'constructing' stamp: 'ar 9/25/2006 13:52'!
addItemSpec: itemSpec target: aTarget
	"Add an item.  The first parameter, itemSpec, is 
		*  either... the symbol $-, to request a separator
		*  or...an array of two or three items:
				(1)  item label
				(2)  either:
						item selector (what will be sent to the target)
					  or
						an array (defining a submenu)
				(3)  optional - balloon-help string"

	| itemName itemValue item |
	itemSpec == #- ifTrue:[^self addSeparator].
	itemName := itemSpec first.
	itemValue := itemSpec second.
	item := self addItem: itemName.
	(itemValue isMemberOf: Array) ifTrue:[
		item subMenu: (TMenuFrame new addList: itemValue target: aTarget).
	] ifFalse:[
		item action: (TMenuAction receiver: aTarget selector: itemValue).
	].
	itemSpec size > 2
		ifTrue: [item helpText: itemSpec third].

	^item! !

!TMenuFrame methodsFor: 'constructing' stamp: 'ar 7/31/2006 11:18'!
addList: itemSpecList target: aTarget
	itemSpecList do:[:itemSpec|
		self addItemSpec: itemSpec target: aTarget.
	].! !

!TMenuFrame methodsFor: 'constructing' stamp: 'ar 8/1/2006 08:42'!
addMenuItem: aMenuItemSpec
	menuItems := menuItems copyWith: aMenuItemSpec.
	self changed.! !

!TMenuFrame methodsFor: 'constructing' stamp: 'ar 7/31/2006 11:18'!
addSeparator
	menuItems size > 0 ifTrue:[menuItems last separator: true].! !

!TMenuFrame methodsFor: 'accessing' stamp: 'ar 8/7/2006 14:00'!
alignment
	"Answer the alignment of this menu:
		#topLeft, #topCenter, #topRight
		#leftCenter, #center, #rightCenter
		#bottomLeft, #bottomCenter, #bottomRight
	"
	^alignment! !

!TMenuFrame methodsFor: 'accessing' stamp: 'ar 8/7/2006 14:01'!
alignment: aSymbol
	"Set the alignment of this menu:
		#topLeft, #topCenter, #topRight
		#leftCenter, #center, #rightCenter
		#bottomLeft, #bottomCenter, #bottomRight
	"
	alignment := aSymbol.
	self changed.! !

!TMenuFrame methodsFor: 'properties' stamp: 'ar 8/1/2006 10:01'!
backgroundColor
	^menuStyle backgroundColor! !

!TMenuFrame methodsFor: 'updating' stamp: 'ar 8/1/2006 08:42'!
changed
	menuForm ifNotNil:[
		menuForm := nil.
		self future updateMenuStructure.
	].! !

!TMenuFrame methodsFor: 'initialize' stamp: 'das 8/11/2006 11:39'!
close
	self selectItemIndex: 0.
	parentMenu ifNotNil:[parentMenu subMenuClosed: self].
	self delete.
"	frameParent ifNotNil:[frameParent removeChild: self]."! !

!TMenuFrame methodsFor: 'events' stamp: 'ar 8/4/2006 17:21'!
closeMenuUnlessFocused: aMenu
	"Close aMenu unless it is focused"
	aMenu isFocused ifFalse:[aMenu close].
! !

!TMenuFrame methodsFor: 'properties' stamp: 'ar 8/1/2006 10:01'!
disabledTextColor
	^menuStyle disabledTextColor! !

!TMenuFrame methodsFor: 'events' stamp: 'ar 7/31/2006 15:38'!
eventMask
	^EventPointerDown + EventPointerOver.! !

!TMenuFrame methodsFor: 'events' stamp: 'ar 9/25/2006 13:43'!
fireSelectedItem: event
	"Fire the selected menu item"
	selectedItem ifNil:[^nil].
	selectedItem fireWithEvent: event. "any action associated with it"
	selectedItem subMenu ifNil:[^self topMenu close]. "after firing"
! !

!TMenuFrame methodsFor: 'accessing' stamp: 'bgf 8/5/2006 17:48'!
hideLabelFrame
	"Remove the 'labelFrame' (the white draggable top bar) from the menu."
	self removeChild:labelFrame! !

!TMenuFrame methodsFor: 'initialize' stamp: 'bgf 8/7/2006 15:17'!
initialize
	| mat |
	super initialize.
	menuStyle := TMenuStyle new.
	selectedItem := nil.
	menuItems := #().
	alignment := #topCenter.
	focused := false.
	labelFrame := TCube new.
	labelFrame extentX: 1 y: 0.1 z: 0.1.
	mat := TMaterial new.
	mat ambientColor:Color white asVectorColor.
	mat diffuseColor:Color white asVectorColor.
	mat fullBright: true.
	labelFrame material: mat.
	self addChild: labelFrame.

	menuForm := TForm new.
	menuForm initialize: nil form: (Form extent: 128@128 depth: 32) fillWhite
		mipmap: true shrinkFit: true extension: nil.
	menuTxtr := TTexture new.
	menuTxtr initializeWithTForm: menuForm.
	menuTxtr material: mat.
	menuTxtr translationX: 0.0 y: 0.1 z: 0.0.

	self addChild: menuTxtr.
	self eventMask: EventPointerDown + EventPointerOver.
! !

!TMenuFrame methodsFor: 'testing' stamp: 'ar 8/4/2006 17:19'!
isFocused
	"Answer whether this menu is in the active menu chain"
	^focused or:[subMenu notNil and:[subMenu isFocused]]! !

!TMenuFrame methodsFor: 'properties' stamp: 'ar 8/1/2006 10:01'!
menuItemFont
	^menuStyle itemFont! !

!TMenuFrame methodsFor: 'accessing' stamp: 'ar 8/1/2006 10:01'!
menuStyle
	^menuStyle! !

!TMenuFrame methodsFor: 'accessing' stamp: 'ar 8/1/2006 10:01'!
menuStyle: aTMenuStyle
	menuStyle := aTMenuStyle.
	self changed.! !

!TMenuFrame methodsFor: 'properties' stamp: 'ar 8/1/2006 10:01'!
normalItemColor
	^menuStyle normalItemColor! !

!TMenuFrame methodsFor: 'properties' stamp: 'ar 8/1/2006 10:01'!
normalTextColor
	^menuStyle normalTextColor! !

!TMenuFrame methodsFor: 'accessing' stamp: 'ar 8/4/2006 17:16'!
parentMenu
	^parentMenu! !

!TMenuFrame methodsFor: 'accessing' stamp: 'ar 8/4/2006 17:17'!
parentMenu: aMenu
	parentMenu := aMenu! !

!TMenuFrame methodsFor: 'events' stamp: 'ar 7/31/2006 16:20'!
pointerDown: event
	event selection frame == labelFrame ifTrue:[
		selectedPoint := event selection point.
		cameraNorm := self inverseGlobalOrientation localPointToGlobal: 
			event selection cameraTransform lookAt negated.
	].! !

!TMenuFrame methodsFor: 'events' stamp: 'ar 8/4/2006 17:18'!
pointerEnter: evt
	focused := true.
	self pointerOver: evt.! !

!TMenuFrame methodsFor: 'events' stamp: 'ar 8/4/2006 17:18'!
pointerLeave: evt
	focused := false.
	subMenu ifNil:[^self selectItemIndex: 0].
	"If an active submenu exists the pointer might enter the submenu. 
	We post a future message here so that if the pointer goes into the 
	subMenu we catch it accordingly"
	(self future: 50) closeMenuUnlessFocused: subMenu.! !

!TMenuFrame methodsFor: 'events' stamp: 'ar 7/31/2006 18:45'!
pointerMove: event
	| pointer delta |
	event selection frame == labelFrame ifTrue:[
		pointer := event makePointer.
		(pointer frame: self pickPlane: selectedPoint normal: cameraNorm) ifTrue:[
			delta := selectedPoint - pointer selectedPoint.
			self translation: (self translation - (self orientation localPointToGlobal: delta)).
		].
	].
	event selection frame == #menuTxtr ifTrue:[self pointerOver: event].! !

!TMenuFrame methodsFor: 'events' stamp: 'ar 8/4/2006 17:38'!
pointerOver: event
	| pt appExtent idx evtFrame |
	evtFrame := event selection frame.
	(evtFrame = menuTxtr) ifTrue: [
		pt := event selection point x @ event selection point y.
		pt := pt / (menuTxtr extent) + 0.5.
		appExtent := menuTxtr extent * 100.
		pt := pt * appExtent.
		pt := pt x @ (appExtent y - pt y).
		idx := (pt y - 3) / (self menuItemFont height + 3) + 1.
		self selectItemIndex: (idx truncated min: menuItems size max: 0).
	].! !

!TMenuFrame methodsFor: 'events' stamp: 'ar 9/25/2006 13:43'!
pointerUp: event
	event selection frame == menuTxtr ifTrue:[
		self fireSelectedItem: event.
	].
! !

!TMenuFrame methodsFor: 'updating' stamp: 'ar 7/31/2006 18:50'!
redraw
	"Redraw the menu"
	| canvas x y font textColor itemWidth itemHeight itemBox checkMarkIcon subMenuIcon item icon baseColor pos |
	menuForm ifNil:[^nil].
	checkMarkIcon := CIconLibrary default at: #CheckMark.
	subMenuIcon := CIconLibrary default at: #SubMenu.
	canvas := CTransformCanvas on: menuForm textureForm.
	canvas fillRectangle: menuForm boundingBox color: self backgroundColor.
	font := self menuItemFont.
	itemWidth := menuFormExtent x.
	itemHeight := font height + 3.
	1 to: menuItems size do:[:i|
		item := menuItems at: i.
		itemBox := 0@(itemHeight * (i-1) + 3) extent: itemWidth@itemHeight.

		"determine text and base color"
		item == selectedItem ifTrue:[
			textColor := self selectedTextColor.
			baseColor := self selectedItemColor.
		] ifFalse:[
			textColor := self normalTextColor.
			baseColor := self normalItemColor.
		].
		item disabled ifTrue:[textColor := self disabledTextColor].

		"figure out text position"
		pos := itemBox leftCenter.
		pos := (pos x + 20) @ (pos y - (font height // 2)).

		"draw background and text"
		baseColor ifNotNil:[canvas fillRectangle: itemBox color: baseColor].
		canvas drawString: item label at: pos font: font color: textColor.

		"draw the icon"
		icon := item icon.
		item checked ifTrue:[icon := checkMarkIcon].
		icon ifNotNil:[
			x := (itemBox left + 10)  - (icon width // 2).
			y := (itemBox top + (itemBox height // 2)) - (icon height // 2).
			canvas paintImage: icon at: x@y.
		].

		item subMenu ifNotNil:[
			icon := subMenuIcon.
			icon ifNotNil:[canvas paintImage: icon 
				at: itemBox rightCenter - ((icon width + 2)@(icon height // 2))].
		].

		item separator ifTrue:[
			canvas fillRectangle: ((itemBox bottomLeft extent: itemBox width @ 1) insetBy: 4@0)
				color: self separatorColor.
		].
	].
	menuForm invalidate: menuForm boundingBox.! !

!TMenuFrame methodsFor: 'events' stamp: 'ar 8/4/2006 17:16'!
selectItemIndex: index
	"Select the item with the given index"
	| selected |
	index = 0 
		ifTrue:[selected := nil]
		ifFalse:[selected := menuItems at: index].
	selected == selectedItem ifTrue:[^nil].
	subMenu ifNotNil:[
		subMenu parentMenu: nil.
		subMenu  close.
	].
	selectedItem := selected.
	selectedItem ifNotNil:[
		selectedItem subMenu ifNotNil:[self showSubMenuOf: selectedItem].
	].
	self redraw.! !

!TMenuFrame methodsFor: 'properties' stamp: 'ar 8/1/2006 10:01'!
selectedItemColor
	^menuStyle selectedItemColor! !

!TMenuFrame methodsFor: 'properties' stamp: 'ar 8/1/2006 10:01'!
selectedTextColor
	^menuStyle selectedTextColor! !

!TMenuFrame methodsFor: 'properties' stamp: 'ar 8/1/2006 10:01'!
separatorColor
	^menuStyle separatorColor! !

!TMenuFrame methodsFor: 'accessing' stamp: 'bgf 8/5/2006 17:49'!
showLabelFrame
	"Restore the 'labelFrame' (the white draggable top bar) to the menu."
	self addChild:labelFrame! !

!TMenuFrame methodsFor: 'events' stamp: 'ar 8/4/2006 17:16'!
showSubMenuOf: item
	"Show the submenu of the given menu item"
	| index |
	frameParent ifNil:[^nil].
	subMenu := item subMenu ifNil:[^nil].
	index := menuItems indexOf: item ifAbsent:[^nil].
	subMenu parentMenu: self.
	self addChild: subMenu.
	subMenu translationX: menuTxtr extent x * 0.5 y: (index-1) * (self menuItemFont height + 3) * -0.01 z: 0.3.! !

!TMenuFrame methodsFor: 'accessing' stamp: 'ar 8/4/2006 17:17'!
subMenu
	^subMenu! !

!TMenuFrame methodsFor: 'accessing' stamp: 'ar 8/4/2006 17:17'!
subMenu: aMenu
	subMenu := aMenu! !

!TMenuFrame methodsFor: 'events' stamp: 'ar 8/4/2006 17:26'!
subMenuClosed: aMenu
	"Notify a parent menu that the given subMenu has just closed"
	subMenu parentMenu: nil.
	subMenu == aMenu ifFalse:[^nil].
	"Deselect current selection, implicitly closing the subMenu too"
	self selectItemIndex: 0.! !

!TMenuFrame methodsFor: 'accessing' stamp: 'ar 8/4/2006 17:30'!
topMenu
	"Answer the top menu in this menu chain"
	^(parentMenu ifNil:[^self]) topMenu! !

!TMenuFrame methodsFor: 'updating' stamp: 'ar 8/7/2006 14:30'!
updateMenuStructure
	"Menu structure has changed; update it"
	| itemFont menuWidth itemWidth menuHeight  delta |
	menuForm ifNotNil:[^nil]. "already up to date"
	itemFont := self menuItemFont.
	menuHeight := (itemFont height + 3) * menuItems size + 3.
	menuWidth := "menuLabel labelWidth max:" 40.
	menuItems do:[:each|
		itemWidth := (itemFont widthOfString: each label) + 40.
		menuWidth := menuWidth max: itemWidth.
	].
	menuFormExtent := (menuWidth@menuHeight) truncated.
	menuForm := TForm new.
	menuForm initialize: nil form: (Form extent: menuFormExtent asLargerPowerOfTwo depth: 32)
		mipmap: true shrinkFit: true extension: nil.
	menuTxtr initializeWithTForm: menuForm.
	menuTxtr aspect: menuFormExtent y / menuFormExtent x asFloat.
	menuTxtr extent: menuFormExtent * 0.01.
	menuTxtr uvScale: (menuFormExtent / menuFormExtent asLargerPowerOfTwo) asFloatPoint.
	"@@@@ ARGH!!!!!! This is evil: Rectangle>>center truncates to whole pixels and therefore:
		(0@0 extent: 1@1) center => 0@0
	which would completely blow the following computation so scale it by 1000."
	delta := ((0@0 extent: menuTxtr extent * 1000.0) perform: alignment) / 1000.0.
	delta := delta -  (menuTxtr extent * 0.5).
	delta := (0.0 - delta x) @ (delta y).
	menuTxtr translationX: delta x y: delta y z: 0.0.
	labelFrame translationX: delta x y: delta y + (menuTxtr extent y * 0.5) z: 0.0.
	labelFrame extentX: menuTxtr extent x+0.1.
	self redraw.! !

TObject subclass: #TMenuItem
	instanceVariableNames: 'label action checked disabled separator icon subMenu'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Croquet-Menus'!

!TMenuItem commentStamp: '<historical>' prior: 0!
(c) 2006 Qwaq Inc. A Croquet in-world menu item.

Instance variables:
	label	<String>			The menu label.
	action	<MessageSend>	The action to take when the item is invoked.
	checked	<Boolean>		If true, the item should be displayed with a checkmark.
	disabled	<Boolean>		If true, the item should be displayed as disabled (grayed out)
	separator	<Boolean>	If true, the item should be followed with a separator
	icon	<Form>			The icon to display for this item.
	subMenu	<TMenu>		The subMenu associated with this item.
!

!TMenuItem methodsFor: 'accessing' stamp: 'ar 7/31/2006 11:25'!
action
	^action! !

!TMenuItem methodsFor: 'accessing' stamp: 'ar 7/31/2006 11:25'!
action: aMessageSend
	action := aMessageSend! !

!TMenuItem methodsFor: 'accessing' stamp: 'ar 7/31/2006 11:25'!
checked
	^checked! !

!TMenuItem methodsFor: 'accessing' stamp: 'ar 7/31/2006 11:25'!
checked: aBool
	checked := aBool! !

!TMenuItem methodsFor: 'accessing' stamp: 'ar 7/31/2006 11:26'!
disabled
	^disabled! !

!TMenuItem methodsFor: 'accessing' stamp: 'ar 7/31/2006 11:26'!
disabled: aBool
	disabled := aBool! !

!TMenuItem methodsFor: 'actions' stamp: 'ar 7/31/2006 15:46'!
fire
	"Fire this menu"
	^action value! !

!TMenuItem methodsFor: 'actions' stamp: 'ar 9/25/2006 13:51'!
fireWithEvent: event
	"Fire this menu item with an optional argument"
	^action fireWithEvent: event! !

!TMenuItem methodsFor: 'accessing' stamp: 'ar 7/31/2006 11:26'!
icon
	^icon! !

!TMenuItem methodsFor: 'accessing' stamp: 'ar 7/31/2006 11:26'!
icon: anIcon
	icon := anIcon.! !

!TMenuItem methodsFor: 'initialize' stamp: 'ar 7/31/2006 11:28'!
initialize
	super initialize.
	label := 'MenuItem'.
	action := nil.
	checked := false.
	disabled := false.
	separator := false.
	icon := nil.
	subMenu := nil.
! !

!TMenuItem methodsFor: 'accessing' stamp: 'ar 7/31/2006 11:25'!
label
	^label! !

!TMenuItem methodsFor: 'accessing' stamp: 'ar 7/31/2006 11:25'!
label: aString
	label := aString! !

!TMenuItem methodsFor: 'accessing' stamp: 'ar 7/31/2006 11:28'!
separator
	^separator! !

!TMenuItem methodsFor: 'accessing' stamp: 'ar 7/31/2006 11:28'!
separator: aBool
	separator := aBool! !

!TMenuItem methodsFor: 'accessing' stamp: 'ar 7/31/2006 11:26'!
subMenu
	^subMenu! !

!TMenuItem methodsFor: 'accessing' stamp: 'ar 7/31/2006 11:26'!
subMenu: aMenu
	subMenu := aMenu! !

TObject subclass: #TMenuStyle
	instanceVariableNames: 'backgroundColor itemFont normalItemColor selectedItemColor normalTextColor selectedTextColor disabledTextColor separatorColor'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Croquet-Menus'!

!TMenuStyle commentStamp: '<historical>' prior: 0!
(c) 2006 Qwaq Inc. Describes style attribues for menus.

Instance variables:
	backgroundColor	<Color>	The background color for the menu.
	itemFont 			<Font>	The font to use for menu items.
	normalItemColor 	<Color>	The background color for items (if any)
	selectedItemColor 	<Color>	The background color for selected items
	normalTextColor 	<Color>	The text color for normal items
	selectedTextColor 	<Color>	The text color for selected items
	disabledTextColor 	<Color>	The text color for disabled items
	separatorColor 		<Color>	The color to use for separators
!

!TMenuStyle methodsFor: 'accessing' stamp: 'ar 8/1/2006 08:35'!
backgroundColor
	^backgroundColor! !

!TMenuStyle methodsFor: 'accessing' stamp: 'ar 8/1/2006 08:35'!
backgroundColor: aColor
	backgroundColor := aColor! !

!TMenuStyle methodsFor: 'accessing' stamp: 'ar 8/1/2006 08:35'!
disabledTextColor
	^disabledTextColor! !

!TMenuStyle methodsFor: 'accessing' stamp: 'ar 8/1/2006 08:35'!
disabledTextColor: aColor
	disabledTextColor := aColor.! !

!TMenuStyle methodsFor: 'initialize' stamp: 'bgf 8/7/2006 15:22'!
initialize
	super initialize.
	itemFont := Preferences standardMenuFont.
	normalItemColor := nil. "use background color"
	selectedItemColor := Color blue.
	normalTextColor := Color black.
	selectedTextColor := Color white.
	disabledTextColor := Color gray.
	separatorColor := Color blue.
	backgroundColor := Color veryLightGray.
! !

!TMenuStyle methodsFor: 'accessing' stamp: 'ar 8/1/2006 08:39'!
itemFont
	^itemFont! !

!TMenuStyle methodsFor: 'accessing' stamp: 'ar 8/1/2006 08:39'!
itemFont: aFont
	itemFont := aFont.! !

!TMenuStyle methodsFor: 'accessing' stamp: 'ar 8/1/2006 08:35'!
normalItemColor
	^normalItemColor! !

!TMenuStyle methodsFor: 'accessing' stamp: 'ar 8/1/2006 08:35'!
normalItemColor: aColor
	normalItemColor := aColor! !

!TMenuStyle methodsFor: 'accessing' stamp: 'ar 8/1/2006 08:35'!
normalTextColor
	^normalTextColor! !

!TMenuStyle methodsFor: 'accessing' stamp: 'ar 8/1/2006 08:36'!
normalTextColor: aColor
	normalTextColor := aColor! !

!TMenuStyle methodsFor: 'accessing' stamp: 'ar 8/1/2006 08:36'!
selectedItemColor
	^selectedItemColor! !

!TMenuStyle methodsFor: 'accessing' stamp: 'ar 8/1/2006 08:36'!
selectedItemColor: aColor
	selectedItemColor := aColor.! !

!TMenuStyle methodsFor: 'accessing' stamp: 'ar 8/1/2006 08:36'!
selectedTextColor
	^selectedTextColor! !

!TMenuStyle methodsFor: 'accessing' stamp: 'ar 8/1/2006 08:36'!
selectedTextColor: aColor
	selectedTextColor := aColor! !

!TMenuStyle methodsFor: 'accessing' stamp: 'ar 8/1/2006 08:36'!
separatorColor
	^separatorColor! !

!TMenuStyle methodsFor: 'accessing' stamp: 'ar 8/1/2006 08:36'!
separatorColor: aColor
	separatorColor := aColor! !

TCube subclass: #TMenuTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Croquet-Menus'!

!TMenuTest commentStamp: '<historical>' prior: 0!
(c) 2006 Qwaq Inc. An example for menus.!

!TMenuTest methodsFor: 'initialize' stamp: 'ar 9/25/2006 13:56'!
initialize
	super initialize.
	eventMask := EventPointerDown.! !

!TMenuTest methodsFor: 'events' stamp: 'das 8/10/2006 11:35'!
pointerDown: event
	"Position it"
	| menu item |
	menu := TMenuFrame new.
	item := menu addItem: 'Item with submenu'.
	item subMenu: ((TMenuFrame new)
		addList: #(
			('Item A' #yourself)
			('Item B' #yourself)
			('Item C' #yourself)
			('Item D' #yourself)
		) target: self;
		yourself).
	menu addSeparator.
	item := menu addItem: 'Checked item'.
	item checked: true.
	item := menu addItem: 'Disabled item'.
	item disabled: true.
	item := menu addItem: 'Item with icon'.
	item icon: (CIconLibrary default at: #DefaultEventIcon).
	menu addSeparator.
	item := menu addItem: 'Item with submenu'.
	item subMenu: ((TMenuFrame new)
		addList: #(
			('Item A' #yourself)
			('Item B' #yourself)
			('Item C' #yourself)
			('Item D' #yourself)
		) target: self;
		yourself).
	menu addSeparator.
	"Add an entire list"
	menu addList: #(
		('Item 1' #yourself)
		('Item 2' #yourself)
		('Item 3' #yourself)
		('Item 4' #yourself)
		('Item 5' #yourself)
		('Item 6' #yourself)
		('Another submenu' (
			('Item A' #yourself)
			('Item B' #yourself)
			('Item C' #yourself)
			('Item D' #yourself)
			('Another submenu' (
				('Item A' #yourself)
				('Item B' #yourself)
				('Item C' #yourself)
				('Item D' #yourself)
				('Another submenu' (
					('Item A' #yourself)
					('Item B' #yourself)
					('Item C' #yourself)
					('Item D' #yourself)
					('Another submenu' (
						('Item A' #yourself)
						('Item B' #yourself)
						('Item C' #yourself)
						('Item D' #yourself)
					))
				))
			))
		))
		-
		('close menu' yourself)
	) target: self.

event shiftPressed ifTrue:["puts menu in replica frame"
	menu translationX: -3 y: 3 z: -3.0.
	event avatar carrying: menu.
] ifFalse:["puts menu next to object"
	menu translation: self globalPosition + event avatar globalPosition * 0.5.
	menu rotationAroundY: event avatar yaw.
	self root addChild: menu.
].! !

MessageSend subclass: #TMenuAction
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Croquet-Menus'!

!TMenuAction commentStamp: '<historical>' prior: 0!
(c) 2006 Qwaq Inc. TMenuAction is a message taking an extra argument (the event firing the action) for evaluation.!

!TMenuAction methodsFor: 'evaluating' stamp: 'ar 9/25/2006 13:51'!
fireWithEvent: anEvent
	"Fire this message with the given event"
	| nArgs args |
	nArgs := selector numArgs.
	args := self arguments ifNil:[#()].
	nArgs = arguments size ifTrue:[
		^receiver perform: selector withArguments: args].
	nArgs = (arguments size + 1) ifTrue:[
		^receiver perform: selector withArguments: (args copyWith: anEvent)].
	"For now, raise an error. We can always extend it later."
	^self error: 'Invalid number of arguments'! !
